package main

import (
	"os"
	"testing"
)

const DEBUG_DB bool = false

// Test the db.ExpireScores
func TestExpireScores(t *testing.T) {
	const testdb = ".space-test.db"

	db := DBData{}
	// Start with empty database
	os.Remove(testdb)

	db.Open(testdb)
	defer db.Close()
	if !DEBUG_DB {
		defer os.Remove(testdb)
	}

	db.User = "Testing"

	// stuff score data into database
	// SaveScore(when int64, date int64, hand int, won int, best int, score int)
	type TData struct {
		when  YMD
		date  YMD
		hand  int
		won   int
		run   int
		score int
	}

	var data []TData = []TData{
		{20230101, 20230101, 1, 0, 8, 750},
		{20230101, 20230101, 2, 0, 8, 780},
		{20230101, 20230101, 3, 1, 5, 980},
		{20230101, 20230102, 1, 1, 9, 945},
		{20230101, 20230102, 2, 1, 6, 985},
		{20230101, 20230102, 3, 0, 10, 700},
		{20230101, 20230115, 1, 0, 7, 600},
		{20230101, 20230115, 2, 0, 7, 730},
		{20230101, 20230115, 3, 1, 5, 1000}}

	var score, best, won int

	for idx := range data {
		db.SaveScore(data[idx].when, data[idx].date, data[idx].hand,
			data[idx].won, data[idx].run, data[idx].score)

		// Calculate the score, best, and won
		score += data[idx].score
		if data[idx].run > best {
			best = data[idx].run
		}
		won += data[idx].won
	}

	// Data loaded .. call Expire!
	var next_month YMD = 20230202 //time.Time = time.Unix(1645929274, 0)
	next_month.SetDay(1)
	db.ExpireScores(next_month)

	// 1. Verify the scores table is empty.
	scores := db.GetScores(5)

	if len(scores) != 0 {
		t.Errorf("Scores not empty after ExpireScores got %d expected 0.\n", len(scores))
	}
	// 2. Verify the months table is correct.
	monthly := db.GetMonthlyScores(5)

	/*
		type MonthlyData struct {
			Date      int64
			User      string
			Days      int
			Hands_Won int
			Best      int
			Score     int
		}
	*/

	if len(monthly) != 1 {
		t.Errorf("Monthly Scores: got %d, expected 1.\n", len(monthly))
	} else {
		var md MonthlyData = MonthlyData{202301, "Testing", 3, won, best, score}

		if monthly[0].Date != md.Date {
			t.Errorf("Date %d, expected %d.\n", monthly[0].Date, md.Date)
		}
		if monthly[0].User != md.User {
			t.Errorf("User %s, expected %s.\n", monthly[0].User, md.User)
		}
		if monthly[0].Days != md.Days {
			t.Errorf("Days %d, expected %d.\n", monthly[0].Days, md.Days)
		}
		if monthly[0].Hands_Won != md.Hands_Won {
			t.Errorf("Hands %d, expected %d.\n", monthly[0].Hands_Won, md.Hands_Won)
		}
		if monthly[0].Best_Run != md.Best_Run {
			t.Errorf("Run %d, expected %d.\n", monthly[0].Best_Run, md.Best_Run)
		}
		if monthly[0].Score != md.Score {
			t.Errorf("Score %d, expected %d.\n", monthly[0].Score, md.Score)
		}
	}

}
